// v8sharp is a community project available under the Microsoft Public License (Ms-PL)
// Code is provided as is and with no warrenty  Use at your own risk
// View the project and the latest code at http://v8sharp.codeplex.com/

#pragma once

#include "v8script.h"
#include "v8engine.h"
#include <v8.h>

using namespace System::Collections::Generic;

class V8ObjectTemplate;

class V8HandleScope;

namespace v8sharp {
	ref class V8RegisteredType;

	ref class V8EngineImpl sealed : V8Engine {
	internal:
		V8EngineImpl();

	public:
		~V8EngineImpl() { Destroy(); }

		virtual void Register(String^ name, Object^ instance) override;

		generic<typename T> 
		virtual void Register(String^ name, T instance) override;

		virtual void Register(String^ name, Object^ instance, Type^ type) override;

		virtual void RegisterFunction(String^ name, Delegate^ function) override;

		generic<typename T> 
		virtual void Register() override;

		virtual void Register(Type^ type) override;

		virtual void Register(Assembly^ assembly) override;

		virtual void Register(Assembly^ assembly, Predicate<Type^>^ typeFilter) override;

		virtual void Register(Assembly^ assembly, String^ nameSpace) override;

		virtual Object^ Execute(String^ script) override;

		virtual Object^ Execute(V8Script^ script) override;

		virtual V8Script^ Compile(String^ script) override;

	private:
		V8ObjectTemplate* objectTemplate;

		V8HandleScope* handleScope;

		List<V8RegisteredType^>^ registeredTypes;

		V8RegisteredType^ EnsureRegistered(Type^ type, bool allowAbstractOrInterface);

		V8RegisteredType^ EnsureRegistered(Type^ type) {
			return EnsureRegistered(type, false);
		}

		void Destroy();
	};
}

class V8HandleScope {
private:
	v8::HandleScope handleScope;
};