// v8sharp is a community project available under the Microsoft Public License (Ms-PL)
// Code is provided as is and with no warrenty  Use at your own risk
// View the project and the latest code at http://v8sharp.codeplex.com/

#include "v8scriptimpl.h"
#include "v8value.h"
#include <vcclr.h>
#include <string.h>

namespace v8sharp {
	String^ V8ScriptImpl::ScriptText::get() {;
		return this->scriptText;
	}

	bool V8ScriptImpl::HasExceptions::get() {
		return this->exceptions != nullptr && this->exceptions->Count > 0;
	}

	IList<Exception^>^ V8ScriptImpl::Exceptions::get() {
		return this->exceptions;
	}

	V8Script^ V8ScriptImpl::Compile(v8::Persistent<v8::Context> context, String^ script) {
		v8::HandleScope handleScope;
		v8::TryCatch tryCatch;

		v8::Context::Scope contextScope(context);
		pin_ptr<const wchar_t> scriptPtr = PtrToStringChars(script);
		
		v8::Handle<v8::String> stringPtr = v8::String::New((const uint16_t*)scriptPtr);
		v8::Handle<v8::Script> compiled = v8::Script::Compile(stringPtr);

		V8ScriptImpl^ v8Script = gcnew V8ScriptImpl();
		v8Script->scriptText = script;
		if (compiled.IsEmpty()) {
			v8::String::Utf8Value error(tryCatch.Exception());
			
			String^ exceptionMessage = gcnew String((const char*)*error);
			v8Script->exceptions = gcnew List<Exception^>(1);
			v8Script->exceptions->Add(gcnew Exception(exceptionMessage));
		} else {			
			v8Script->compiledScript = new V8ScriptWrapper(v8::Persistent<v8::Script>::New(compiled));
		}

		return v8Script;
	}

	Object^ V8ScriptImpl::Execute(v8::Persistent<v8::Context> context) {
		v8::HandleScope handleScope;
		v8::Context::Scope contextScope(context);
		v8::TryCatch tryCatch;

		v8::Handle<v8::Value> result = this->compiledScript->GetScript()->Run();
		if (tryCatch.HasCaught()) {
			v8::String::Utf8Value error(tryCatch.Exception());
			
			String^ exceptionMessage = gcnew String((const char*)*error);
			throw gcnew Exception(exceptionMessage);
		}

		return V8ValueWrapper::Wrap(result);
	}

	void V8ScriptImpl::Destroy() {
		if (this->compiledScript) {
			delete this->compiledScript;
			this->compiledScript = NULL;
		}
	}
}


V8ScriptWrapper::V8ScriptWrapper(v8::Persistent<v8::Script> script) {
	this->script = script;
}

V8ScriptWrapper::~V8ScriptWrapper() {
	this->script.Dispose();
}

v8::Persistent<v8::Script> V8ScriptWrapper::GetScript() {
	return this->script;
}