// v8sharp is a community project available under the Microsoft Public License (Ms-PL)
// Code is provided as is and with no warrenty  Use at your own risk
// View the project and the latest code at http://v8sharp.codeplex.com/

#pragma once

#include <v8.h>
#include <vcclr.h>

using namespace System;
using namespace System::Collections::Generic;

class V8ObjectTemplate;
class V8InstanceList;
class V8MethodList;
class V8ObjectAccessor;
class V8ExternalWrapper;

namespace v8sharp {	
	ref class V8RegisteredType {
	public:
		V8RegisteredType(System::Type^ type, V8ObjectTemplate* rootTemplate);

		!V8RegisteredType() { Destroy(); }

		~V8RegisteredType() { Destroy(); }

	internal:
		property System::Type^ Type {
			System::Type^ get();
		}

		V8ObjectTemplate* GetInternal();

		virtual void Destroy();

		void Register(String^ name, Object^ instance);

	private:
		System::Type^ type;

		V8ObjectTemplate* objectTemplate;

		V8ObjectTemplate* rootTemplate;

		V8InstanceList* instances;

		V8MethodList* methodList;

		static V8ObjectTemplate* CreateObjectTemplate(V8RegisteredType^ type, V8ObjectTemplate* rootTemplate);

		static v8::Handle<v8::Object> GetObject(String^ nameSpace, V8ObjectTemplate* rootTemplate);
	};
}

class V8ObjectTemplate {
public:
	V8ObjectTemplate(v8::Handle<v8::ObjectTemplate> objTemplate = v8::Handle<v8::ObjectTemplate>());

	~V8ObjectTemplate();

	v8::Handle<v8::ObjectTemplate> GetTemplate();

	v8::Persistent<v8::Context> GetContext();

private:
	v8::Handle<v8::ObjectTemplate> objectTemplate;

	v8::Persistent<v8::Context> context;
};

class V8InstanceList {
public:
	V8InstanceList();

	~V8InstanceList();

	gcroot<String^> Name;

	gcroot<Object^> Instance;

	V8ExternalWrapper* NativeInstance;

	bool IsFilled;

	V8InstanceList* Next;
};

class V8MethodInvocation {
public:
	V8MethodInvocation() { }

	~V8MethodInvocation() { }

	gcroot<String^> Name;

	gcroot<v8sharp::V8RegisteredType^> RegisteredType;
};

class V8MethodList {
public:
	V8MethodList() { }

	~V8MethodList();

	V8MethodInvocation* Instance;

	V8MethodList* Next;
};